package graphics;


import java.awt.Dimension;
import java.io.IOException;
import java.util.ArrayList;

import core.ClientNetworkThread;
import core.LectureGame;


/**
 * 
 * Class interfacing to the Core package. Contains the high level methods of the Graphics package.
 * @author Terje fsdahl
 */
public class GraphicsFacade {
	
	private ArrayList<String> loggedInUsers;
	ArrayList<FallingText> loginQueue;
	private int screenState;
	private QuestionGraphics questionGraphics;
	private String lectureCode;
	private Renderer renderer;
	private int currentModeActivated = -1;
	private LectureGame lega;
	private AudioSample attentionSound;
	
	private GameMode currentGameMode = null;
	
	/**
	 * 
	 * Instantiates variables, mousehandler, keyboardhandler and the OpenGL graphics.
	 * 
	 * @param fullscreen Fullscreen or windowed version.
	 * @param dimension Preferred resolution of the window/fullscreen.
	 * @param lega The LectureGame object.
	 */
    public GraphicsFacade(boolean fullscreen, Dimension dimension,LectureGame lega) {
        renderer = new Renderer(this);
    	loggedInUsers = new ArrayList<String>();
    	loginQueue = new ArrayList<FallingText>();
    	questionGraphics = new QuestionGraphics();
    	screenState = Constants.IDLE_LOGIN_STATE;   
    	lectureCode = "";
    	this.lega = lega;
    	try {
			attentionSound = new  AudioSample(ResourceRetriever.getResourceAsStream("pause.wav"));
		} catch (IOException e) {
			e.printStackTrace();
		}
    	
        GLDisplay neheGLDisplay = GLDisplay.createGLDisplay("The Lecture Game", fullscreen, dimension);

        InputHandler inputHandler = new InputHandler(renderer, neheGLDisplay);
        MouseHandler mouseHandler = new MouseHandler(renderer, neheGLDisplay);
        neheGLDisplay.addGLEventListener(renderer);
        neheGLDisplay.addKeyListener(inputHandler);
        neheGLDisplay.addMouseListener(mouseHandler);
        neheGLDisplay.setNetwork(getNetworkConnection());
        neheGLDisplay.start();
    }
 

    /**
     * Called whenever a user logs in. This is only used to display a welcome message on the screen.
     * The actual model of respondents are kept in other variables and updated in other methods.
     * 
     * @param userName
     */
	public void userLoggedIn(String userName){
    	loggedInUsers.add(userName);
		loginQueue.add(new FallingText(userName,-5.0f,10.0f));
		System.out.println("Bruker lagt til");
	}

	/**
	 * 
	 * Called whenever a new quesiton is received. The mtehod sleeps Constants.DELAY_FOR_MOBILE_SYNC before updating
	 * the question to compromise for slower GPRS/3G links, than the wired/WIFI connection of this client.
	 * 
	 * @param question The actual question string
	 * @param alternatives An array of alternatives as Strings
	 * @param timelimit how many seconds the students will have to answer the question
	 * @param correctAlternativeIndex The index of the correct alternative
	 * @param controlNumber an optional number for game specific processing. Might be the time of reception of the answers 
	 * for future implemntation, or the summarized score.
	 */
	public void newQuestion(String question,ArrayList<String> alternatives, int timelimit, int correctAlternativeIndex, int controlNumber){
		attentionSound.play();
		
		System.out.println("Question popped. Waiting!");
		//force delay to compensate for sync algorithm.
		try {
			Thread.sleep(Constants.DELAY_FOR_MOBILE_SYNC);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		questionGraphics.setQuestion(question);
		questionGraphics.setAlternatives(alternatives);
		questionGraphics.setTimeLimit(timelimit);
		questionGraphics.setCorrectAlternativeIndex(correctAlternativeIndex);
		renderer.setQuestionInitialized(false);
		if(controlNumber==1)
			renderer.setLastQuestion(true);
		else if(controlNumber==2){
			renderer.setLastRound(true);
			renderer.setLastQuestion(true);
		}
		
	}

	
	public int getGameMode(){
		return currentModeActivated; 
	}
	
	public QuestionGraphics getQuestionGraphics() {
		return questionGraphics;
	}

	public void setQuestionGraphics(QuestionGraphics questionGraphics) {
		this.questionGraphics = questionGraphics;
	}

	public int getScreenState() {
		return screenState;
	}

	/**
	 * 
	 * Called whenever the state of the program is changed in order for the Core package to know which
	 * state the program is in.
	 * 
	 * @param screenState
	 */
	public void setScreenState(int screenState) {
		this.screenState = screenState;
	}

	/**
	 * Returns the representation of names to be displayed in the welcome message.
	 * 
	 * @return
	 */
	public ArrayList<FallingText> getLoginQueue() {
		return loginQueue;
	}

	public void setLoginQueue(ArrayList<FallingText> loginQueue) {
		this.loginQueue = loginQueue;
	}

	public String getLectureCode() {
		return lectureCode;
	}

	public void setLectureCode(String lectureCode) {
		this.lectureCode = lectureCode;
	}

	public ArrayList<String> getLoggedInUsers() {
		return loggedInUsers;
	}

	public void setLoggedInUsers(ArrayList<String> loggedInUsers) {
		this.loggedInUsers = loggedInUsers;
	}

	public Renderer getRenderer() {
		return renderer;
	}
	
	/**
	 * 
	 * Routs the result to the renderer instance for graphical processing.
	 * 
	 * @param respondents
	 */
	public void returnResults(Respondent[] respondents){
		renderer.returnResults(questionGraphics,respondents);
	}

	/**
	 * Updates the state of the current Mode.
	 * @param mode
	 */
	public void currentModeActivated(int mode) {
		currentModeActivated = mode;
		
	}

	public ClientNetworkThread getNetworkConnection(){
		return lega.getNetworkConnection();
	}
	
	/**
	 * Sends the specified string to the Server.
	 * @param string
	 */
	public void netSend(String string) {
		lega.netSend(string);
		
	}

/**
 *  
 *  Routs the round result to the Server.renderer instance for graphical processing.
 * @param respondents
 */
	public void returnRoundResults(Respondent[] respondents) {
		renderer.returnRoundResults(respondents);
		
	}

/**
 * Routs the final high score list to the renderer for graphical processing.
 * @param respondents
 */
	public void returnGameResults(Respondent[] respondents) {
		renderer.returnGameResults(respondents);
		
	}
	
}
